using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;
namespace STPP
{
    public class Control : PowerBase, Messages.IUseContinuous
    {
        private bool Choking;
        private GameObject Head;
        private float Distance;
        private float val = 0f;

        public override void Start()
        {
            //We set our power activation type (Head,Chest,Hand)
            Activation = "Hand";

            //Always keep this
            DisablesGrips = false;
            base.Start();
        }
        
        //This is the actual power part of your power, this function will trigger when the activation limb is used.
        public override void UsePower()
        {
            this.Head = null;
            Choking = false;
            //This if statement exists to check if your power is selected and enabled in the power menu
            if (Enabled)
            {
                if (!this.Choking)
                {
                    this.val = 1;
                    var cast = Physics2D.CircleCast(this.transform.position, ModAPI.PixelSize * 30f, -this.transform.up, 15f, LayerMask.GetMask("Objects")); //we use circle cast because being slightly off would be annoying.
                    if (cast.transform != null && cast.transform.root != this.transform.root)
                    {
                            this.Head = cast.transform.gameObject;
                            this.Choking = true;
                            this.Distance = cast.distance;
                    }
                }
            }
        }

        public void UseContinuous(ActivationPropagation prop)
        {
            //This if statement exists to check if your power is selected and enabled in the power menu
            if (Enabled)
            {
             this.Choking = true;
            }
        }

        public void Update()
        {
            if (this.Head != null)
            {
            if (this.Choking)
            {
                if (Input.GetKeyUp(KeyCode.F))
                {
                this.Choking = false;
                this.Head.GetComponent<Rigidbody2D>().velocity = this.gameObject.GetComponent<Rigidbody2D>().velocity;
                }
                this.Head.GetComponent<PhysicalBehaviour>().rigidbody.MovePosition(this.transform.TransformPoint(0f, -this.Distance, 0f));
                this.Head.GetComponent<PhysicalBehaviour>().rigidbody.MoveRotation(this.transform.rotation * Quaternion.Euler(0f, 0f, -90f * this.transform.localScale.x));
            }
            }
        }

        /// <summary>
        /// Set up the power on a character.
        /// </summary>
        /// <param name="body">The selected limb of the user.</param>
        /// <param name="person">The PersonBehaviour of the user. (body.Person)</param>
        /// <param name="Instance">The overall instance of the user. ( body.Person.gameObject)</param>
        public static void SetUpPower(LimbBehaviour body, PersonBehaviour person,Sprite powerIcon = null,string powerName = "Control")
        {
            //This checks if the person already has the power, if they don't we give it to them (duplicates are bad) NOTE:make sure to change the class to your new power class
            if (!body.GetComponent<Control>() || body.GetComponent<Control>().PowerName != powerName)
            {
                //Adds your new power to the person NOTE:make sure to change the class to your new power class
                var Power = body.gameObject.AddComponent<Control>();
                
                //This sets the power's icon, make sure you add the your new icon to the ResourceStorage NOTE:make sure to change the class to your new power class
                if(powerIcon == null)
                {
                    Power.PowerIcon = STPP.IconTelekinesis;
                }
                else
                {
                    Power.PowerIcon = powerIcon;
                }

                Power.PowerName = powerName;

                body.gameObject.AddComponent<UseEventTrigger>().Action = () =>
                {
                    //This triggers your power NOTE:make sure to change the class to your new power class
                    Power.UsePower();
                };
            }
        }
    }
}